#ifndef FREENECTRGBDEVICE_HXX
#define FREENECTRGBDEVICE_HXX

#include "libfreenect.hpp"
#include <cmath>
#include <QMutex>


//Die eigentliche Kinect (erbt von der abstrakten Klasse FreenectDevice)
class FreenectRGBDevice : public Freenect::FreenectDevice 
{
public:
	//Erstellen eines Kinect Geräts
	FreenectRGBDevice(freenect_context *_ctx, int _index)
	:	Freenect::FreenectDevice(_ctx, _index),
		m_new_rgb_frame(false),
		m_new_depth_frame(false)
	{
		
		m_buffer_depth	= (uint8_t*)  malloc(FREENECT_VIDEO_RGB_SIZE*sizeof(uint8_t));
		m_buffer_video	= (uint8_t*)  malloc(FREENECT_VIDEO_RGB_SIZE*sizeof(uint8_t));
		m_gamma			= (uint16_t*) malloc(FREENECT_VIDEO_RGB_SIZE*sizeof(uint16_t));
		
		for( unsigned int i = 0 ; i < 2048 ; i++) {
			float v = i/2048.0;
			v = std::pow(v, 3)* 6;
			m_gamma[i] = v*6*256;
		}
	}
	
	//Wenn ein Exemplar zerstört wird...
	~FreenectRGBDevice()
	{
		delete[] m_buffer_depth;
		delete[] m_buffer_video;
		delete[] m_gamma;
	}
	
	//Die Callback-Funktion, die bei neu vorliegenden Videodaten aufgerufen wird
	//ACHTUNG: Nemals direkt aufrufen!!!
	void VideoCallback(void* _rgb, uint32_t /*timestamp*/) {
		m_rgb_mutex.lock();
			memcpy(m_buffer_video,  _rgb,  FREENECT_VIDEO_RGB_SIZE*sizeof(uint8_t));
			m_new_rgb_frame = true;
		m_rgb_mutex.unlock();
	};
	
	//Die Callback-Funktion, die bei neu vorliegenden Tiefendaten aufgerufen wird
	//ACHTUNG: Nemals direkt aufrufen!!!
	void DepthCallback(void* _depth, uint32_t /*timestamp*/) {
		m_depth_mutex.lock();
			for( unsigned int i = 0 ; i < FREENECT_FRAME_PIX ; i++) {
				int pval = m_gamma[((uint16_t*)_depth)[i]];
				int lb = pval & 0xff;
				switch (pval>>8) {
					case 0:
						m_buffer_depth[3*i+0] = 255;
						m_buffer_depth[3*i+1] = 255-lb;
						m_buffer_depth[3*i+2] = 255-lb;
						break;
					case 1:
						m_buffer_depth[3*i+0] = 255;
						m_buffer_depth[3*i+1] = lb;
						m_buffer_depth[3*i+2] = 0;
						break;
					case 2:
						m_buffer_depth[3*i+0] = 255-lb;
						m_buffer_depth[3*i+1] = 255;
						m_buffer_depth[3*i+2] = 0;
						break;
					case 3:
						m_buffer_depth[3*i+0] = 0;
						m_buffer_depth[3*i+1] = 255;
						m_buffer_depth[3*i+2] = lb;
						break;
					case 4:
						m_buffer_depth[3*i+0] = 0;
						m_buffer_depth[3*i+1] = 255-lb;
						m_buffer_depth[3*i+2] = 255;
						break;
					case 5:
						m_buffer_depth[3*i+0] = 0;
						m_buffer_depth[3*i+1] = 0;
						m_buffer_depth[3*i+2] = 255-lb;
						break;
					default:
						m_buffer_depth[3*i+0] = 0;
						m_buffer_depth[3*i+1] = 0;
						m_buffer_depth[3*i+2] = 0;
						break;
				}
			}
			m_new_depth_frame = true;
		m_depth_mutex.unlock();
	}
	
	//Über diese Funktionen kann man sich Kopien der Kameradaten holen.
	//Der buffer-pointer muss auf ein bereits alloziertes Feld zeigen!
	bool getRGB(uint8_t *buffer) {
		m_rgb_mutex.lock();
		if(m_new_rgb_frame) {
			memcpy(buffer, m_buffer_video, FREENECT_VIDEO_RGB_SIZE*sizeof(uint8_t));
			m_new_rgb_frame = false;
			m_rgb_mutex.unlock();
			return true;
		} else {
			m_rgb_mutex.unlock();
			return false;
		}
	}
	
	//Über diese Funktionen kann man sich Kopien der Tiefendaten holen.
	//Der buffer-pointer muss auf ein bereits alloziertes Feld zeigen!
	bool getDepth(uint8_t* buffer) {
		m_depth_mutex.lock();
		if(m_new_depth_frame) {
			memcpy(buffer, m_buffer_depth, FREENECT_VIDEO_RGB_SIZE*sizeof(uint8_t));
			m_new_depth_frame = false;
			m_depth_mutex.unlock();
			return true;
		} else {
			m_depth_mutex.unlock();
			return false;
		}
	}
	
	//die die verwendete Buffergröße an andere weiter
	int getBufferSize()
	{
		return FREENECT_VIDEO_RGB_SIZE;
	}
	
private:
	uint8_t*  m_buffer_depth;
	uint8_t*  m_buffer_video;
	uint16_t* m_gamma;
	
	QMutex m_rgb_mutex;
	QMutex m_depth_mutex;
	
	bool m_new_rgb_frame;
	bool m_new_depth_frame;
};

#endif